__doc__ = """
DataObject
Data Objects are simple Zope objects with properties. When they are accessed
via URL traversal, they delegate their web rendering to a different object -
the "template". This object is found by evaluating the property "template".

The Data Object is used mostly by giving it it meaningful properties. However,
these properties can also be filtered before rendering, by creating a second
property with the same name plus a '_filter' suffix; this property should be a
string and contain the name of a method in the acquisition path, which accepts
to be called with a single parameter.

For example: you create a property named 'title_filter' and give it the value
'capitalize'; then you create a Python Method in the acquisition path named
'capitalize', with one parameter (s), consisting of just
'return string.capitalize (s)'.

TODO: test it with a DTMLMethod/Document as the template
"""
__version__ = '0.3.0'

import Globals, Acquisition, AccessControl.Role, OFS.SimpleItem, OFS.PropertyManager, DateTime
import string
import utils

_dobj_deprecated_attrs = ()
_dobj_prototype = {'template': '', 'title': '',
	}

class DataObject (
	OFS.SimpleItem.Item,
	Globals.Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	OFS.PropertyManager.PropertyManager
	):
	"""DataObject class
	"""

	#################################################
	### Zope class attributes

	name = 'DataObject'
	id = 'DataObject'
	meta_type = 'Data Object'
	icon = 'misc_/HiperDom/DataObject.gif'
	# Tell DocumentTemplate that we accept its calling conventions...
	isDocTemp = 1
	# ...then tell HiperDom that we prefer ours
	isXmlTemp = 1

	_properties = (
			{'id': 'title', 'type': 'string', 'mode': 'w'},
			{'id': 'template', 'type': 'string', 'mode': 'w'},
		)

	manage_options = (
			{'label':'Properties', 'action':'manage_propertiesForm',
			 'help': ('OFSP', 'Properties.stx')},
			{'label':'View', 'action':''},
			{'label':'Security', 'action':'manage_access',
			 'help': ('OFSP', 'Security.stx')},
			{'label':'Ownership', 'action':'manage_owner',
			 'help': ('OFSP', 'Ownership.stx')},
		)


	__ac_permissions__ = (
			('View management screens',
			 ('manage_tabs',)),
			('Change Permissions',
			 ('manage_access',)),
			('Manage Properties',
			 ('manage_addProperty', 'manage_editProperties',
			 'manage_delProperties', 'manage_changeProperties',)),
			('View',
			 ('', 'index_html', '__call__',)),
			('Know the secrets of life',
			 ('manage_dict',)),
			)


	#################################################
	## Instance constructors

	def __init__ (self, id):
		"Initialize the class instance"
		self.id = id
		for name, value in _dobj_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)

	def __setstate__ (self, state):
		"Initialize when loading from the ZODB"
		Globals.Persistent.__setstate__ (self, state)
		for name, value in _dobj_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		for name in _dobj_deprecated_attrs:
			if hasattr (self, name):
				delattr (self, name)


	#################################################
	## Management interface
	# (None)


	#################################################
	## Externally accessible, but not from the manage
	## interface
	
	def index_html (self, REQUEST, **kw):
		"Render the data into a webpage"
		return apply (self.__str__, (self, REQUEST), kw)


	#################################################
	## Externally accessible from Python/DTML only
	
	def __str__ (self, context=None, REQUEST=None, **kw):
		"Render the data"
		if context is None:
			context = self
		if REQUEST is None:
			if hasattr (context, 'REQUEST'):
				REQUEST = context.REQUEST
			else:
				REQUEST = {}
		# apply filters
		kw = kw.copy()
		for name, value in self.propertyItems():
			if hasattr (self, name + '_filter'):
				kw[name] = self.aq_acquire (getattr (self, name + '_filter')) (value)
		# evaluate the template
		return utils.evaluate (self.template, context, REQUEST, kw)

	def __call__ (self, context=None, REQUEST=None, **kw):
		"Render the data given a context (?)"
		# DocumentTemplate calls us with (None, context)
		if context is None and hasattr (REQUEST, 'has_key') and REQUEST.has_key ('REQUEST'):
			context = REQUEST
			REQUEST = context['REQUEST']
		# FIXME: this will lose our properties when context != self...
		return apply (self.__str__, (context, REQUEST), kw)


	#################################################
	## Methods for internal consumption
	


	#################################################
	## Debugging stuff, in the "what not even God
	## should do" category
	def manage_dict (self, REQUEST=None):
		"Debugging method - meaning is obvious"
		return `self.__dict__`

manage_addDataObjectForm = Globals.HTMLFile ('DataObjectAdd', globals ())

def manage_addDataObject (self, id, REQUEST=None):
	"Add a DataObject to the current ObjectManager"
	obj = DataObject (id)
	assert obj, "Could not create object!"
	obj.title = REQUEST['title']
	obj.template = REQUEST['template']
	self._setObject (id, obj)
	if REQUEST is not None:
		return self.manage_main (self, REQUEST)
