__doc__ = 'Strategy objects to process HiperDom directives'

import utils

preserveDirectives = '   ' # special marker

class _strategy:
	def __init__ (self, context, REQUEST, template, kw={}):
		self.r = REQUEST
		self.kw = kw
		self.context = context
		self.template = template

class assetHunter (_strategy):
	def __init__ (self, REQUEST, parentStrategy):
		self.r = REQUEST
		self.parent = parentStrategy

	def do_id (self, template, element, specialData):
		"""Process the data and add the results to specialData with
		the name specified in the hdom:id directive
		"""
		name = element.getAttribute ('hdom:id')
		if specialData.has_key (preserveDirectives):
			# We don't want to remove the hdom:id, probably because
			# we're running from inside a textExpander
			wkcopy = element
		else:
			# remove the hdom:id attribute
			wkcopy = element.cloneNode (deep = 1)
			wkcopy.removeAttribute ('hdom:id')
		specialData[name] = template._processElement (wkcopy, self.parent).toxml()
		return element

class textExpander (_strategy):
	def do_text (self, template, element, specialData, stripDirectives=0):
		"""Replace the contents (childNodes) of the element by the
		result of evaluating the identifier passed as the value of
		the hdom:text directive.
		But first, hunt for any assets in the subtree to be discarded
		(assets are elements with an hdom:id attribute); the assets
		are added to the namespace when the identifier is evaluated.
		Finally, here we don't want to strip the Hdom directives
		unless explicitly told so
		"""
		name = element.getAttribute ('hdom:text')
		wkcopy = element.cloneNode (deep = 0)
		assets = self.kw.copy()
		if stripDirectives:
			wkcopy.removeAttribute ('hdom:text')
		else:
			assets[preserveDirectives] = 1
		template._processElement (element, assetHunter (self.r, self), assets)
		if assets.has_key (preserveDirectives):
			del assets[preserveDirectives]

		value = utils.evaluate (name, self.context, self.r, assets)
		
		wkcopy._node.children.append (utils.alreadyRendered (str (value)))
		return wkcopy

class fullHdom (textExpander):
	def do_text (self, template, element, specialData):
		"""Replace the contents (childNodes) of the element by the
		result of evaluating the identifier passed as the value of
		the hdom:text attribute.
		But first, hunt for any assets in the subtree to be discarded
		(assets are elements with an hdom:id attribute); the assets
		are added to the namespace when the identifier is evaluated.
		"""
		# We just call the same method from textExpander, telling it
		# to discard the HiperDom directives
		return textExpander.do_text (self, template, element, specialData, 1)

	def do_dummy (self, template, element, specialData):
		"""Discard this element"""
		return None

	def do_condition (self, template, element, specialData):
		"""Evaluate the name given as the directive's value, then leave the
		contents untouched if the name evaluates to true and discard the
		element if it evaluates to false.
		"""
		name = element.getAttribute ('hdom:condition')
		wkcopy = element.cloneNode (deep = 1)
		wkcopy.removeAttribute ('hdom:condition')

		if utils.evaluate (name, self.context, self.r, self.kw):
			return wkcopy
		else:
			return None

	def do_loop (self, template, element, specialData):
		"""Evaluate the name given as the directive's value, then process
		the contents of the element once for every item of the result.
		It is an error for the result not to be a sequence object.
		"""
		name = element.getAttribute ('hdom:loop')
		wkcopy = element.cloneNode (deep = 1)
		wkcopy.removeAttribute ('hdom:loop')
		fragment = element.ownerDocument.createDocumentFragment()
		namespace = self.kw.copy()

		for item in utils.evaluate (name, self.context, self.r, self.kw):
			namespace['sequence_item'] = item
			if hasattr (item, '__of__'):
				newStrategy = fullHdom (item.__of__ (self.context), self.r, template, namespace)
			else:
				# FIXME: not sure
				newStrategy = fullHdom (self.context, self.r, template, namespace)
			child = template._processElement (wkcopy, newStrategy)
			fragment._node.children.append (child._node)
		return fragment

	def do_method (self, template, element, specialData):
		"""Replace the whole element by a new element resulting from
		evaluating the identifier passed as the value of the hdom:method
		directive.
		The method will be called with a special parameter: element_data,
		which is the xml.dom.Element object.		
		It may return a dictionary with some special conventions (keys are
		attributes, a special key '<text>' is the content), or a string
		(in this case the node is replaced by the alreadyRendered string),
		or a new xml.dom Element object.
		"""
		name = element.getAttribute ('hdom:method')
		wkcopy = element.cloneNode (deep = 1)
		assets = self.kw.copy()
		wkcopy.removeAttribute ('hdom:text')
		wkcopy.__allow_access_to_unprotected_subobjects = 1
		assets['element_data'] = wkcopy
		result = utils.evaluate (name, self.context, self.r, assets)
 		if result is None or type (result) == type ('') or hasattr (result, 'toxml'):
			return result
		elif type (result) == type ({}):
			newtext = ''
			if result.has_key ('<text>'):
				newtext = result['<text>']
				del result['<text>']
			for name in wkcopy.attributes.keys():
				wkcopy.removeAttribute (name)
			for name, value in result.items():
				wkcopy.setAttribute (name, value)
			for child in wkcopy.childNodes[:]:
				wkcopy.removeChild (child)
			if newtext:
				wkcopy._node.children.append (utils.alreadyRendered (newtext))
			return wkcopy
		else:
			raise TypeError, 'invalid object returned from hdom:method'

