__doc__ = """
HiperDom Template objects
HiperDom Templates are XML page templates - meaning, they can generate XML
documents if they're processed in a Zope (for example) context. The primary
use for them is generating xHTML webpages.

HiperDom works by recognizing special XML attributes (called Directives),
marked with a "hdom:" XML namespace. For example,
<title hdom:text="docTitle">foo</title> will replace "foo" by the value
resulting from evaluating the variable, property or function "docTitle".
"""
__version__ = '0.3.0'

import Globals, Acquisition, AccessControl.Role, OFS.SimpleItem, OFS.PropertyManager, DateTime
import string
import xml.sax.saxexts, xml.dom.sax_builder #, html_builder
import HiperDomStrategies, utils

_hdom_deprecated_attrs = ()
_hdom_prototype = {'HTML': 0, 'text': '<EmptyTemplate/>',
		'_dom': None, 'title': 'HiperDom Template', 'content_type': '',
	}

class HiperDomTemplate (
	OFS.SimpleItem.Item,
	Globals.Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	OFS.PropertyManager.PropertyManager
	):
	"""HiperDom Template class
	__call__ and __str__ render the template;
	index_html renders the template and sets the proper content-type on RESPONSE;
	manage_getTemplate returns the source, and sets content-type;
	text_expand processes only hdom:text directives, leaving them in place and
		respecting hdom:id assets
	The template object is "meaningful" in the namespace, or in other words,
		it's parallel to a DTML Document, not a DTML Method
	"""

	#################################################
	### Zope class attributes

	name = 'HiperDomTemplate'
	id = 'HiperDomTemplate'
	meta_type = 'HiperDom Template'
	icon = 'misc_/HiperDom/HiperDomTemplate.gif'
	# Tell DocumentTemplate that we accept its calling conventions...
	isDocTemp = 1
	# ...then tell HiperDom that we prefer ours
	isXmlTemp = 1

	_properties = (
			{'id': 'HTML', 'type': 'boolean', 'mode': 'w'},
			{'id': 'title', 'type': 'string', 'mode': 'w'},
			{'id': 'content_type', 'type': 'string', 'mode': 'w'},
		)

	manage_options = (
			{'label':'Edit', 'action':'manage_main',
			 'help': ('HiperDom', 'TemplateEdit.stx')},
			{'label':'Expand', 'action':'manage_expand',
			 'help': ('HiperDom', 'TemplateExpand.stx')},
			{'label':'View', 'action':''},
			{'label':'Properties', 'action':'manage_propertiesForm',
			 'help': ('OFSP', 'Properties.stx')},
			{'label':'Security', 'action':'manage_access',
			 'help': ('OFSP', 'Security.stx')},
			{'label':'Ownership', 'action':'manage_owner',
			 'help': ('OFSP', 'Ownership.stx')},
		)


	__ac_permissions__ = (
			('View management screens',
			 ('manage_tabs', 'manage_main', 'manage_expand',
			  'manage_getTemplate', 'text_expand')),
			('Change Permissions',
			 ('manage_access',)),
			('Change HiperDom Templates',
			 ('manage_edit', 'PUT')),
			('Manage Properties',
			 ('manage_addProperty', 'manage_editProperties',
			 'manage_delProperties', 'manage_changeProperties',)),
			('View',
			 ('', 'index_html', '__call__',)),
			('Know the secrets of life',
			 ('manage_dict',)),
			)


	#################################################
	## Instance constructors

	def __init__ (self, id):
		"Initialize the class instance"
		self.id = id
		for name, value in _hdom_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		if not self._dom:
			self._updateDom()

	def __setstate__ (self, state):
		"Initialize when loading from the ZODB"
		Globals.Persistent.__setstate__ (self, state)
		for name, value in _hdom_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		for name in _hdom_deprecated_attrs:
			if hasattr (self, name):
				delattr (self, name)
		if not self._dom:
			self._updateDom()


	#################################################
	## This does the "Narower/Wider/Taller/Shorter"
	## manage_main textarea magic
	## (shamelessly copied from OFS.DTMLMethod)

	_size_changes = {
		'Bigger': (5,5),
		'Smaller': (-5,-5),
		'Narrower': (0,-5),
		'Wider': (0,5),
		'Taller': (5,0),
		'Shorter': (-5,0),
	}

	def _er (self, text, submit, dtpref_cols, dtpref_rows, REQUEST):
		dr, dc = self._size_changes[submit]
        
		rows = max (1, string.atoi (dtpref_rows) + dr)
		cols = max (40, string.atoi (dtpref_cols) + dc)
		e = (DateTime.DateTime ('GMT') + 365).rfc822()
		resp = REQUEST['RESPONSE']
		resp.setCookie ('dtpref_rows', str (rows), path='/', expires=e)
		resp.setCookie ('dtpref_cols', str (cols), path='/', expires=e)
		return self.manage_main (self, REQUEST, text=text, dtpref_cols=cols, dtpref_rows=rows)


	#################################################
	## Management interface

	manage_main = Globals.HTMLFile ('HiperDomTemplateEdit', globals())
	manage_expand = Globals.HTMLFile ('HiperDomTemplateExpand', globals())

	def manage_edit (self, text, submit='Change', dtpref_cols='50',
                    dtpref_rows='20', REQUEST=None):
		"Edit the template."
		if self._size_changes.has_key (submit):
 			return self._er (text, submit, dtpref_cols, dtpref_rows, REQUEST)
		
		self.text = text
		self._updateDom()

		if REQUEST is not None:
			return self.manage_main (self, REQUEST,
				manage_tabs_message = 'Template changed.')


	#################################################
	## Externally accessible, but not from the manage
	## interface
	
	def index_html (self, REQUEST, **kw):
		"Render the template into a webpage"
		result = apply (self.__str__, (self, REQUEST), kw)
		self._setContentType (REQUEST)
		return result

	def manage_getTemplate (self, REQUEST=None):
		"""Return the template
		It is rendered from the dom tree instead of
		just self.text, so that we return a normalized
		version (or XMLized if it was HTML)"""
		if REQUEST:
			self._setContentType (REQUEST)
		return self._dom.toxml()

	def PUT (self, REQUEST, RESPONSE):
		"Change the template from an HTTP PUT"
		self.dav__init (REQUEST, RESPONSE)
		self.text = REQUEST.get ('BODY', '')
		self._updateDom()
		return RESPONSE


	#################################################
	## Externally accessible from Python/DTML only
	
	def __str__ (self, context=None, REQUEST=None, **kw):
		"Render the template"
		if context is None:
			context = self
		if REQUEST is None:
			if hasattr (context, 'REQUEST'):
				REQUEST = context.REQUEST
			else:
				REQUEST = {}
		return self._processElement (self._dom, HiperDomStrategies.fullHdom (context, REQUEST, self, kw)).toxml()

	def __call__ (self, context=None, REQUEST=None, **kw):
		"Render the template given a context (?)"
		# DocumentTemplate calls us with (None, context)
		if context is None and hasattr (REQUEST, 'has_key') and REQUEST.has_key ('REQUEST'):
			context = REQUEST
			REQUEST = context['REQUEST']
		# FIXME: how should I use the context? Where does Acquisition fit
		# into this?
		return apply (self.__str__, (context, REQUEST), kw)

	def text_expand (self, REQUEST=None, **kw):
		"""Process only hdom:text directives, leaving them in place and
		respecting hdom:id assets
		"""
		if not REQUEST:
			REQUEST=self.REQUEST
		return self._processElement (self._dom, HiperDomStrategies.textExpander (self, REQUEST, self, kw)).toxml()


	#################################################
	## Methods for internal consumption
	
	def _updateDom (self):
		"Update the internal DOM tree from the text"
		if 0 and self.HTML:
			# This doesn't work yet, because HtmlBuilderis based on sgmllib
			# and it cripples attributes with colons in their names.
			b = html_builder.HtmlBuilder (ignore_mismatched_end_tags = 1)
			b.feed (self.text)
			b.close()
			self._dom = b.document
			b.document = None
			b.current_element = None
		else:
			p = xml.sax.saxexts.make_parser()
			b = xml.dom.sax_builder.SaxBuilder()
			p.setDocumentHandler(b)
			p.feed(self.text)
			self._dom = b.document
			p.close()
	
	def _setContentType (self, REQUEST):
		"Set the content-type"
		# FIXME: we want more robustness, like checking if the
		# first element in the DOM is an <html> and if we have a
		# Content-type property
		if self.content_type:
			REQUEST.RESPONSE.setHeader ('Content-type', self.content_type)
			return
		if self.HTML or string.lower (self._dom.childNodes[0].nodeName) == 'html':
			# Zope's default content-type (HTML)
		 	return
		REQUEST.RESPONSE.setHeader ('Content-type', 'text/xml')

	def _processElement (self, element, strategy, specialData={}):
		"""hdom:process one element (and its children) using this strategy
		returns the new, processed Node object (or an utils.AlreadyRendered
 		pseudo-Node if the strategy returned a string)
 		specialData may be used to "fish" for data in the tree; for
 		example, it is used to gather assets marked with hdom:id
 		before gobbling an hdom:text element
		"""
		# skip the Document node if for some bizarre reason it exists
		if element.nodeType == xml.dom.core.DOCUMENT_NODE:
			return self._processElement (element.childNodes[0], strategy)
		# also skip alreadyRendered stuff
		elif element.nodeType == 'alreadyRendered':
			return element
		# handle hdom: attributes
		attrs = element.attributes
		wkcopy = element
		if attrs:
			for key in filter (lambda k: k[:5] == 'hdom:',
			  attrs.keys()):
				if hasattr (strategy, 'do_' + key[5:]):
					wkcopy = getattr (strategy, 'do_' + key[5:]) (self, wkcopy, specialData)
					if not wkcopy:
						return None
					if type (wkcopy) == type (''):
						return (utils.alreadyRenderedNode (
						 utils.alreadyRendered (wkcopy), element.ownerDocument))
		# we don't want to modify the upstream element object
		children = wkcopy.childNodes
		wkcopy = wkcopy.cloneNode (deep = 0)
		# process the children
		for child in children:
			newChild = self._processElement (child, strategy, specialData)
			if newChild:
				# this level of kludge shouldn't be necessary...
				# sadly, we can't just append an AlreadyRenderedNode
				wkcopy._node.children.append (newChild._node)
		return wkcopy


	#################################################
	## Debugging stuff, in the "what not even God
	## should do" category
	def manage_dict (self, REQUEST=None):
		"Debugging method - meaning is obvious"
		return `self.__dict__`

manage_addHiperDomTemplateForm = Globals.HTMLFile ('HiperDomTemplateAdd', globals ())

def manage_addHiperDomTemplate (self, id, REQUEST=None):
	"Add an HiperDomTemplate to the current ObjectManager"
	obj = HiperDomTemplate (id)
	assert obj, "Could not create template!"
	obj.title = REQUEST['title']
	self._setObject (id, obj)
	if REQUEST is not None:
		return self.manage_main (self, REQUEST)
