__doc__ = """
ZUnit tests for the HiperDomTemplate class
"""

from Products import ZUnit
import HiperDomTemplate
import string

class creationTest (ZUnit.TestCase):
	def runTest (self):
		"Try to create a template"
		ctx = self.ZopeContext
		ctx.REQUEST.set ('title', 'Testing creation')
		HiperDomTemplate.manage_addHiperDomTemplate (ctx, 'created', ctx.REQUEST)
		self.failUnless ('created' in ctx.objectIds(), 'Couldn\'t create HiperDom Template')
		try:
			template = ctx.created.manage_getTemplate()
			self.failUnless (template[0] == '<' and template[-1] == '>', 'HiperDom Template not properly initialized')
		except:
			self.fail ('HiperDom Template not properly initialized, couldn\'t render source template:' + ZUnit.SubTraceback())

class objectTests (ZUnit.TestCase):
	def setUp (self):
		ctx = self.ZopeContext
		self.r = ctx.REQUEST
		self.r.set ('title', 'Testing HiperDom')
		HiperDomTemplate.manage_addHiperDomTemplate (ctx, 'template', ctx.REQUEST)
		self.t = ctx.template

	def tearDown (self):
		self.ZopeContext.manage_renameObject ('template',
			'template%02d' % len (self.ZopeContext.objectIds()))

	def test01_manage_main (self):
		"Try to access manage_main"
		try:
			page = self.t.manage_main (self.t, self.r)
		except:
			self.fail ('Couldn\'t view management screen (manage_main):' + ZUnit.subTraceback())
		self.failUnless (string.find (page, '>&lt;EmptyTemplate/&gt;</TEXTAREA>') > 0,
			'Management screen corrupted (manage_main)')
	
	def test02_manage_edit (self):
		"Try to edit the template via manage_edit"
		try:
			self.t.manage_edit (text='<foo><bar name="Madame Satan" city="Sao Paulo"/></foo>')
		except:
			self.fail ('Couldn\'t edit template:' + ZUnit.subTraceback())
		# manage_getTemplate() is too unpredictable due to python's XML library being
		# a work in progress
		#t = self.t.manage_getTemplate()
		t = self.t.text
		self.failUnless (t == '<foo><bar name="Madame Satan" city="Sao Paulo"/></foo>',
			'Something creepy happened with my template:\n		 ' + t)

	def makeRenderTest (self, substring, invert=None):
		"Try to render the template, then check if the results contain 'substring'"
		try:
			t = str (self.t)
		except:
			self.fail ('Couldn\'t render template:' + ZUnit.subTraceback())
		if invert:
			test = self.failIf
		else:
			test = self.failUnless
		test (string.find (t, substring) >= 0,
			'The template was rendered incorrectly:\n		 ' +
			string.join (string.split (t, '\n'), '\n		 '))
	
	def test03_renderSimpeTemplate (self):
		"Try to render a template that doesn't contain any HiperDom directives"
		self.test02_manage_edit()
		self.makeRenderTest ('Madame Satan')
	
	def test04_renderDummyDirective (self):
		"Try to use the hdom:dummy directive"
		self.t.manage_edit (text='<foo><bar hdom:dummy="1"/></foo>')
		self.makeRenderTest ('<bar', 1)
	
	def test05_renderTextDirective (self):
		"Try to use the hdom:text directive"
		self.t.manage_edit (text='<foo hdom:text="bar"></foo>')
		self.t.bar = 'Madame Satan'
		self.makeRenderTest ('<foo>Madame Satan</foo>')
	
	def test06_renderTrueConditionDirective (self):
		"Try to use the hdom:condition directive with a true condition"
		self.t.manage_edit (text='<foo><bar hdom:condition="yes"/></foo>')
		self.t.yes = 1
		self.makeRenderTest ('<bar')
	
	def test07_renderFalseConditionDirective (self):
		"Try to use the hdom:condition directive with a false condition"
		self.t.manage_edit (text='<foo><bar hdom:condition="no"/></foo>')
		self.t.no = 0
		self.makeRenderTest ('<bar', 1)
	
	def test08_renderLoopDirective (self):
		"Try to use the hdom:loop directive to loop over a list of 3 ints"
		self.t.manage_edit (text='<foo hdom:loop="list"><bar hdom:text="sequence_item"/></foo>')
		self.t.list = [1, 2, 3]
		try:
			t = str (self.t)
		except:
			self.fail ('Couldn\'t render template:' + ZUnit.subTraceback())
		self.failUnless ((string.find (t, '<bar>1') >= 0) and
			(string.find (t, '<bar>2') >= 0) and (string.find (t, '<bar>3') >= 0),
			'The template was rendered incorrectly:\n		 ' +
			string.join (string.split (t, '\n'), '\n		 '))
	
	def test09_renderMethodDirectiveReturningNone (self):
		"Try to use the hdom:method directive with a method that returns None"
		self.t.manage_edit (text='<foo><bar hdom:method="none"/></foo>')
		def return_None (context, REQUEST, **assets):
			if not (assets.has_key ('element_data')):
				raise AssertionError, 'Method for hdom:method invoked incorrectly by HiperDom'
		self.t.none = return_None
		self.makeRenderTest ('<bar', 1)
	
	def test10_renderMethodDirectiveReturningString (self):
		"Try to use the hdom:method directive with a method that returns a string"
		self.t.manage_edit (text='<foo><bar hdom:method="somethod"/></foo>')
		def somethod (context, REQUEST, **assets):
			if not (assets.has_key ('element_data')):
				raise AssertionError, 'Method for hdom:method invoked incorrectly by HiperDom'
			return 'It worked.'
		self.t.somethod = somethod
		self.makeRenderTest ('It worked.')
	
	def test11_renderMethodDirectiveReturningXML (self):
		"Try to use the hdom:method directive with a method that returns a xml node object"
		self.t.manage_edit (text='<foo><bar hdom:method="somethod"/></foo>')
		def somethod (context, REQUEST, **assets):
			if not (assets.has_key ('element_data')):
				raise AssertionError, 'Method for hdom:method invoked incorrectly by HiperDom'
			elem = assets['element_data']
			elem._node.name = 'success'
			return elem
		self.t.somethod = somethod
		self.makeRenderTest ('<success')
	
	def test12_renderMethodDirectiveReturningDict (self):
		"""Try to use the hdom:method directive with a method that returns a
		dictionary representing an empty element"""
		self.t.manage_edit (text='<foo><bar hdom:method="somethod"/></foo>')
		def somethod (context, REQUEST, **assets):
			if not (assets.has_key ('element_data')):
				raise AssertionError, 'Method for hdom:method invoked incorrectly by HiperDom'
			return {'name': 'Madame Satan'}
		self.t.somethod = somethod
		# I'd prefer '<bar name="Madame Satan"/>' but xml.dom is broken :-/
		self.makeRenderTest ("<bar name='Madame Satan' />")
	
	def test13_renderMethodDirectiveReturningDictWithText (self):
		"""Try to use the hdom:method directive with a method that returns a
		dictionary representing an element containing text"""
		self.t.manage_edit (text='<foo><bar hdom:method="somethod"/></foo>')
		def somethod (context, REQUEST, **assets):
			if not (assets.has_key ('element_data')):
				raise AssertionError, 'Method for hdom:method invoked incorrectly by HiperDom'
			return {'<text>': 'Madame Satan'}
		self.t.somethod = somethod
		self.makeRenderTest ("<bar>Madame Satan</bar>")
	
	def test14_renderTextDirectiveWithAssets (self):
		"Try to use the hdom:text directive containing hdom:id assets"
		self.t.manage_edit (text='<foo hdom:text="somethod"><bar hdom:id="bar">Madame Satan</bar></foo>')
		def somethod (context, REQUEST, **assets):
			if not (assets.has_key ('bar')):
				raise AssertionError, 'Asset lost!'
			return assets['bar']
		self.t.somethod = somethod
		self.makeRenderTest ("<bar>Madame Satan</bar>")

def creator():
	suite = ZUnit.TestSuite()
	suite.addTest (creationTest())
	suite.addTest (ZUnit.makeSuite (objectTests))
	return suite
